@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: CDDL-1.0
 /*
  * CDDL HEADER START
  *
@@ -6,7 +7,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -93,8 +94,8 @@ uu_list_pool_create(const char *name, size_t objsize,
 
 	(void) pthread_mutex_init(&pp->ulp_lock, NULL);
 
-	pp->ulp_null_list.ul_next_enc = UU_PTR_ENCODE(&pp->ulp_null_list);
-	pp->ulp_null_list.ul_prev_enc = UU_PTR_ENCODE(&pp->ulp_null_list);
+	pp->ulp_null_list.ul_next = &pp->ulp_null_list;
+	pp->ulp_null_list.ul_prev = &pp->ulp_null_list;
 
 	(void) pthread_mutex_lock(&uu_lpool_list_lock);
 	pp->ulp_next = next = &uu_null_lpool;
@@ -110,10 +111,8 @@ void
 uu_list_pool_destroy(uu_list_pool_t *pp)
 {
 	if (pp->ulp_debug) {
-		if (pp->ulp_null_list.ul_next_enc !=
-		    UU_PTR_ENCODE(&pp->ulp_null_list) ||
-		    pp->ulp_null_list.ul_prev_enc !=
-		    UU_PTR_ENCODE(&pp->ulp_null_list)) {
+		if (pp->ulp_null_list.ul_next != &pp->ulp_null_list ||
+		    pp->ulp_null_list.ul_prev != &pp->ulp_null_list) {
 			uu_panic("uu_list_pool_destroy: Pool \"%.*s\" (%p) has "
 			    "outstanding lists, or is corrupt.\n",
 			    (int)sizeof (pp->ulp_name), pp->ulp_name,
@@ -202,7 +201,7 @@ uu_list_create(uu_list_pool_t *pp, void *parent, uint32_t flags)
 	}
 
 	lp->ul_pool = pp;
-	lp->ul_parent_enc = UU_PTR_ENCODE(parent);
+	lp->ul_parent = parent;
 	lp->ul_offset = pp->ulp_nodeoffset;
 	lp->ul_debug = pp->ulp_debug || (flags & UU_LIST_DEBUG);
 	lp->ul_sorted = (flags & UU_LIST_SORTED);
@@ -217,11 +216,11 @@ uu_list_create(uu_list_pool_t *pp, void *parent, uint32_t flags)
 
 	(void) pthread_mutex_lock(&pp->ulp_lock);
 	next = &pp->ulp_null_list;
-	prev = UU_PTR_DECODE(next->ul_prev_enc);
-	lp->ul_next_enc = UU_PTR_ENCODE(next);
-	lp->ul_prev_enc = UU_PTR_ENCODE(prev);
-	next->ul_prev_enc = UU_PTR_ENCODE(lp);
-	prev->ul_next_enc = UU_PTR_ENCODE(lp);
+	prev = next->ul_prev;
+	lp->ul_next = next;
+	lp->ul_prev = prev;
+	next->ul_prev = lp;
+	prev->ul_next = lp;
 	(void) pthread_mutex_unlock(&pp->ulp_lock);
 
 	return (lp);
@@ -250,11 +249,11 @@ uu_list_destroy(uu_list_t *lp)
 	}
 
 	(void) pthread_mutex_lock(&pp->ulp_lock);
-	UU_LIST_PTR(lp->ul_next_enc)->ul_prev_enc = lp->ul_prev_enc;
-	UU_LIST_PTR(lp->ul_prev_enc)->ul_next_enc = lp->ul_next_enc;
+	lp->ul_next->ul_prev = lp->ul_prev;
+	lp->ul_prev->ul_next = lp->ul_next;
 	(void) pthread_mutex_unlock(&pp->ulp_lock);
-	lp->ul_prev_enc = UU_PTR_ENCODE(NULL);
-	lp->ul_next_enc = UU_PTR_ENCODE(NULL);
+	lp->ul_prev = NULL;
+	lp->ul_next = NULL;
 	lp->ul_pool = NULL;
 	uu_free(lp);
 }
@@ -507,14 +506,20 @@ uu_list_walk(uu_list_t *lp, uu_walk_fn_t *func, void *private, uint32_t flags)
 	}
 
 	if (lp->ul_debug || robust) {
-		uu_list_walk_t my_walk;
+		uu_list_walk_t *my_walk;
 		void *e;
 
-		list_walk_init(&my_walk, lp, flags);
+		my_walk = uu_zalloc(sizeof (*my_walk));
+		if (my_walk == NULL)
+			return (-1);
+
+		list_walk_init(my_walk, lp, flags);
 		while (status == UU_WALK_NEXT &&
-		    (e = uu_list_walk_next(&my_walk)) != NULL)
+		    (e = uu_list_walk_next(my_walk)) != NULL)
 			status = (*func)(e, private);
-		list_walk_fini(&my_walk);
+		list_walk_fini(my_walk);
+
+		uu_free(my_walk);
 	} else {
 		if (!reverse) {
 			for (np = lp->ul_null_node.uln_next;
